﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna;
using Microsoft.Xna.Framework;

namespace VillainmadEngine
{
    /// <summary>
    /// Used to run an animation.
    /// </summary>
    public partial struct Animation
    {
        #region Fields
        /// <summary>
        /// Filename of the texture to use as the spritesheet.
        /// Load this separately in the Stage or GameState!
        /// </summary>
        public readonly string Texture;
        /// <summary>
        /// Number of steps to delay before switching to the next one. If 0,
        /// each frame will last for a single step; if 1, each one will last
        /// for 2 steps, and so on.
        /// </summary>
        public readonly int FrameDelay;
        /// <summary>
        /// Source rectangles for the animation-frames.
        /// </summary>
        private readonly Rectangle[] SourceRects;
        /// <summary>
        /// Origins for the rectangles. If the array is empty, defaults to the
        /// center of each source rectangle.
        /// </summary>
        private readonly Vector2[] Origins;
        #endregion

        #region Constructor: String, Int, Rectangles, Vectors
        /// <summary>
        /// Creates a new animation with the specified data.
        /// </summary>
        /// <param name="Texture">Filename of the texture to use as the spritesheet.
        /// Load this separately in the Stage or GameState!</param>
        /// <param name="FrameDelay">Number of steps to delay before switching to the 
        /// next one. If 0, each frame will last for a single step; if 1, each one 
        /// will last for 2 steps, and so on.</param>
        /// <param name="SourceRects">Source rectangles for the animation-frames.</param>
        /// <param name="Origins">Origins for the rectangles. If the array is empty, 
        /// defaults to the center of each source rectangle.</param>
        public Animation(string Texture, int FrameDelay, Rectangle[] SourceRects,
            Vector2[] Origins)
        {
            this.Texture = Texture;
            this.FrameDelay = FrameDelay;
            if (SourceRects.Length != Origins.Length && Origins.Length != 0)
                throw new ArgumentException("Origins must be empty or the same length" +
                    "as SourceRects!");
            this.SourceRects = SourceRects;
            if (Origins.Length == 0)
            {
                this.Origins = new Vector2[SourceRects.Length];
                for (int i = 0; i < SourceRects.Length; i++)
                    this.Origins[i] = new Vector2(SourceRects[i].Width / 2,
                        SourceRects[i].Height / 2);
            }
            else this.Origins = Origins;
        }
        #endregion

        #region Constructor: String, Int, Rectangles
        /// <summary>
        /// Creates a new animation with the specified data, with the origins at the
        /// center of each rectangle.
        /// </summary>
        /// <param name="Texture">Filename of the texture to use as the spritesheet.
        /// Load this separately in the Stage or GameState!</param>
        /// <param name="FrameDelay">Number of steps to delay before switching to the 
        /// next one. If 0, each frame will last for a single step; if 1, each one 
        /// will last for 2 steps, and so on.</param>
        /// <param name="SourceRects">Origins for the rectangles. If the array is empty, 
        /// defaults to the center of each source rectangle.</param>
        public Animation(string Texture, int FrameDelay, Rectangle[] SourceRects)
            : this(Texture, FrameDelay, SourceRects, new Vector2[] { })
        {
        }
        #endregion

        #region CurFrame
        /// <summary>
        /// Gets the current animation-frame, given the specified step.
        /// </summary>
        /// <param name="Step">Number of steps since the animation began.</param>
        /// <returns></returns>
        public int CurFrame(int Step)
        {
            return (int)Math.Truncate((double)(Step % Length) / 
                (FrameDelay + 1));
        }
        #endregion
    }
}
