﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace VillainmadEngine.Configuration
{
    internal struct ScreenResolution
    {
        /// <summary>
        /// Base width of the playable screen.
        /// </summary>
        public const int ScreenWidth = 640;
        /// <summary>
        /// Base height of the playable screen.
        /// </summary>
        public const int ScreenHeight = 480;

        /// <summary>
        /// True if the current resolution is fullscreen; false otherwise.
        /// </summary>
        public readonly bool FullScreen;
        /// <summary>
        /// The size of the playable screen is multiplied by this amount.
        /// </summary>
        public readonly float Multiplier;

        /// <summary>
        /// Width of the window/screen of the resolution.
        /// </summary>
        public readonly int ResWidth;
        /// <summary>
        /// Height of the window/screen of the resolution.
        /// </summary>
        public readonly int ResHeight;

        /// <summary>
        /// If the playable width is less than the 
        /// </summary>
        public int XOffset
        {
            get { return (ResWidth - Width) / 2; }
        }
        /// <summary>
        /// Rectangle, representing the 
        /// </summary>
        public Rectangle ScreenRect
        {
            get { return new Rectangle(XOffset, 0, Width, Height); }
        }
        /// <summary>
        /// Width of the playable screen.
        /// </summary>
        public int Width
        {
            get { return (int)(Multiplier * (float)ScreenWidth); }
        }
        /// <summary>
        /// Height of the playable screen.
        /// </summary>
        public int Height
        {
            get { return (int)(Multiplier * (float)ScreenHeight); }
        }


        private ScreenResolution(int resWidth, int resHeight)
        {
            FullScreen = true;
            ResWidth = resWidth;
            ResHeight = resHeight;

            Multiplier = 1F;
            for (float multiplier = 1.5F; 
                multiplier * ScreenHeight < ResHeight; 
                multiplier += 0.5F)
            {
                Multiplier = multiplier;
            }
        }

        private ScreenResolution(float multiplier)
        {
            FullScreen = false;
            Multiplier = multiplier;
            ResWidth = (int)(Multiplier * (float)ScreenWidth);
            ResHeight = (int)(Multiplier * (float)ScreenHeight);
        }

        /// <summary>
        /// Returns a string representation of the resolution, in the form of
        /// "Fullscreen/Windowed," the resolution, and widescreen proportion (if
        /// any).
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            string retString = FullScreen ? "Fullscreen" : "Windowed";

            retString += " " + ResWidth + "x" + ResHeight;

            if ((float)ResHeight / ResWidth == 0.625f) retString += " (16:10)";
            else if ((float)ResHeight / ResWidth == 0.5625f) retString += " (16:9)";
            else if ((float)ResHeight / ResWidth == 0.8) retString += " (5:4)";

            return retString;
        }

        public static ScreenResolution[] Resolutions
        {
            get { return _resolutions; }
        }

        private static readonly ScreenResolution[] _resolutions = new ScreenResolution[]
        {
            new ScreenResolution(1F),
            new ScreenResolution(1.5F),
            new ScreenResolution(2F),
            new ScreenResolution(640, 480),
            new ScreenResolution(1280, 800),
            new ScreenResolution(1280, 1024),
            new ScreenResolution(1366, 768),
            new ScreenResolution(1440, 900),
        };
    }
}
