﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace VillainmadEngine
{
    /// <summary>
    /// Static class which handles drawing of objects.
    /// </summary>
    public static partial class Drawing
    {
        private static SpriteBatch spriteBatch
        {
            get { return DataStealer.SpriteBatch; }
        }

        private static void DrawTest()
        {
            if (!DataStealer.IsDrawing) throw new AccessViolationException(
                "You can only draw elements within a drawing method.");
        }

        private static Texture2D Texture2DGet(string filename)
        {
            if (filename == null) throw new ArgumentNullException("Texture cannot be null.");
            if (!DataStealer.Texture2DList.ContainsKey(filename))
                throw new KeyNotFoundException("Texture \"gfx\\" + filename + 
                    "\" not loaded.");
            return DataStealer.Texture2DList[filename];
        }

        private static Vector2 OriginGet(Vector2? origin, Rectangle? sourceRectangle,
            Texture2D tex)
        {
            if (origin != null) return origin.Value;

            if (sourceRectangle == null) return tex.Center();
            else return new Vector2(sourceRectangle.Value.Width / 2,
                sourceRectangle.Value.Height / 2);

        }

        #region Draw: Texture, Rectangle-pos, Color
        /// <summary>
        /// Draws a sprite with the specified texture, rectangle, and color.
        /// </summary>
        /// <param name="texture">Filename of the texture, in the "textures"
        /// directory in Content.</param>
        /// <param name="destRectangle">Destination rectangle of the sprite.</param>
        /// <param name="color">The color to tint the sprite.</param>
        public static void Draw(string texture, Rectangle destRectangle, 
            Color color)
        {
            SetBlend(DefaultBlend);
            Texture2D tex = Texture2DGet(texture);

            spriteBatch.Draw(tex, destRectangle, color);
        }
        #endregion

        #region Draw: Texture, Vector2-pos, Color
        /// <summary>
        /// Draws a sprite with the specified texture, position, and tint.
        /// </summary>
        /// <param name="texture">Filename of the texture, in the "textures"
        /// directory in Content.</param>
        /// <param name="position">Vector position of the texture's center.</param>
        /// <param name="color">The color to tint the texture.</param>
        public static void Draw(string texture, Vector2 position, Color color)
        {
            SetBlend(DefaultBlend);
            Texture2D tex = Texture2DGet(texture);

            spriteBatch.Draw(tex, position, null, color, 0F, 
                tex.Center(), 1F, 
                SpriteEffects.None, 0F);
        }
        #endregion

        #region Draw: Texture, DestRectangle, SourceRectangle, Color
        /// <summary>
        /// Draws a sprite with the specified texture, destination rectangle,
        /// optional source rectangle, and color.
        /// </summary>
        /// <param name="texture">Filename of the texture, in the "textures"
        /// directory in Content.</param>
        /// <param name="destRectangle">Destination rectangle of the sprite.</param>
        /// <param name="sourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">The color to tint the texture.</param>
        public static void Draw(string texture, Rectangle destRectangle,
            Rectangle? sourceRectangle, Color color)
        {
            SetBlend(DefaultBlend);

            spriteBatch.Draw(Texture2DGet(texture), destRectangle, 
                sourceRectangle, color);
        }
        #endregion

        #region Draw: Texture, Vector2-pos, SourceRectangle, Color
        /// <summary>
        /// Draws a sprite with the specified texture, position, optional 
        /// source-rectangle, and tint.
        /// </summary>
        /// <param name="texture">Filename of the texture, in the "textures"
        /// directory in Content.</param>
        /// <param name="position">Vector position of the texture's center.</param>
        /// <param name="sourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">The color to tint the sprite.</param>
        public static void Draw(string texture, Vector2 position,
            Rectangle? sourceRectangle, Color color)
        {
            SetBlend(DefaultBlend);
            Texture2D tex = Texture2DGet(texture);

            spriteBatch.Draw(tex, position, sourceRectangle, color, 0F,
                tex.Center(), 1F, SpriteEffects.None, 0F);
        }
        #endregion

        #region Draw: Texture DestRectangle SrcRectangle Color Rot Origin Effects Blend
        /// <summary>
        /// Draws a sprite with the specified texture, destination rectangle,
        /// optional source-rectangle, tint, rotation, origin, and effects.
        /// </summary>
        /// <param name="texture">Filename of the texture, in the "textures"
        /// directory in Content.</param>
        /// <param name="destRectangle">Destination rectangle of the sprite.</param>
        /// <param name="sourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">The color to tint the sprite.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the sprite around its center.</param>
        /// <param name="origin">The sprite's origin/offset from the upper-left
        /// corner. If null, uses the center of the texture.</param>
        /// <param name="effects">Effects to apply.</param>
        /// <param name="blend">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        public static void Draw(string texture, Rectangle destRectangle, 
            Rectangle? sourceRectangle, Color color, float rotation, Vector2? origin,
            SpriteEffects effects, BlendState blend)
        {
            SetBlend(blend);

            Texture2D tex = Texture2DGet(texture);

            spriteBatch.Draw(tex, destRectangle, sourceRectangle,
                color, rotation, OriginGet(origin, sourceRectangle, tex),
                effects, 0);
        }
        #endregion

        #region Draw: Texture Vectpos SrcRect Color Rot Origin FlScale Effect Blend
        /// <summary>
        /// Draws a sprite with the specified texture, position, optional
        /// soruce-rectangle, color, rotation, origin, scale, and effects.
        /// </summary>
        /// <param name="texture">Filename of the texture, in the "textures"
        /// directory in Content.</param>
        /// <param name="position">Vector position of the texture's center.</param>
        /// <param name="sourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">The color to tint the sprite.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the sprite around its center.</param>
        /// <param name="origin">The sprite's origin/offset from the upper-left
        /// corner. If null, uses the center of the texture.</param>
        /// <param name="scale">Scale factor; put 1F for full size.</param>
        /// <param name="effects">Effects to apply.</param>
        /// <param name="blend">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        public static void Draw(string texture, Vector2 position, 
            Rectangle? sourceRectangle, Color color, float rotation, Vector2? origin,
            float scale, SpriteEffects effects, BlendState blend)
        {
            SetBlend(blend);

            Texture2D tex = Texture2DGet(texture);

            spriteBatch.Draw(tex, position, sourceRectangle,
                color, rotation, OriginGet(origin, sourceRectangle, tex), scale, 
                effects, 0F);
        }
        #endregion

        #region Draw: Texture Vectpos SrcRect Color Rot Origin VectScale Effect Blend
        /// <summary>
        /// Draws a sprite with the specified texture, position, optional
        /// soruce-rectangle, color, rotation, origin, scale, and effects.
        /// </summary>
        /// <param name="texture">Filename of the texture, in the "textures"
        /// directory in Content.</param>
        /// <param name="position">Vector position of the texture's center.</param>
        /// <param name="sourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">The color to tint the sprite.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the sprite around its center.</param>
        /// <param name="origin">The sprite's origin/offset from the upper-left
        /// corner. If null, uses the center of the texture.</param>
        /// <param name="scale">Scale factor; put Vector2.One for full size.</param>
        /// <param name="effects">Effects to apply.</param>
        /// <param name="blend">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        public static void Draw(string texture, Vector2 position,
            Rectangle? sourceRectangle, Color color, float rotation, Vector2? origin,
            Vector2 scale, SpriteEffects effects, BlendState blend)
        {
            SetBlend(blend);

            Texture2D tex = Texture2DGet(texture);
            spriteBatch.Draw(tex, position, sourceRectangle,
                color, rotation, OriginGet(origin, sourceRectangle, tex), 
                scale, effects, 0F);
        }
        #endregion

    }
}