﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace VillainmadEngine
{
    /// <summary>
    /// A type of GameEntity which is controlled by the player.
    /// </summary>
    public class PlayerCharacter : GameEntity
    {
        /// <summary>
        /// The layer the player-character exists in.
        /// </summary>
        public sealed override int Layer
        {
            get { return Stage.PlayerLayer; }
        }

        /// <summary>
        /// The player character's speed per frame.
        /// </summary>
        public virtual float Speed
        {
            get { return 3F; }
        }

        /// <summary>
        /// True if the character can currently move; false otherwise.
        /// </summary>
        public bool CanMove = true;

        /// <summary>
        /// Called automatically to enact input.
        /// </summary>
        public void Input()
        {
            Input(CurrentInput.InputList, CurrentInput.InputPressed,
                CurrentInput.InputReleased);
        }

        /// <summary>
        /// <para>Called automatically to enact input. Default form moves the player
        /// along according to their speed.</para>
        /// <para>You do it like this: each of the arguments are flag-enums of 
        /// the GameInput type. To determine if a given button is currently down/has
        /// just been pressed/has just been released, use a boolean comparison to
        /// the list.</para>
        /// <para>For instance, if you want to check if the Fire1 button has just been
        /// pressed, you would do an if-statement in the form of "if((InputPressed &amp;
        /// GameInput.Fire1) != 0) ..." or "if((InputPressed &amp; GameInput.Fire1) == 
        /// GameInput.Fire1) ..."</para>
        /// </summary>
        /// <param name="InputList">List of buttons etc. which are currently pressed
        /// down.</param>
        /// <param name="InputPressed">List of buttons etc. which have just been 
        /// pressed and were not previously down.</param>
        /// <param name="InputReleased">List of buttons etc. which have just been 
        /// released and are no longer down.</param>
        protected virtual void Input(GameInput InputList, GameInput InputPressed,
            GameInput InputReleased)
        {
            if (!CanMove) return;

            if ((InputList & GameInput.DUp) != 0) Velocity.Y = -1;
            else if ((InputList & GameInput.DDown) != 0) Velocity.Y = 1;
            else Velocity.Y = 0;

            if ((InputList & GameInput.DLeft) != 0) Velocity.X = -1;
            else if ((InputList & GameInput.DRight) != 0) Velocity.X = 1;
            else Velocity.X = 0;

            if (Velocity.Y != 0 && Velocity.X != 0) Velocity.Normalize();

            //if (InputPressed != 0) System.Windows.Forms.MessageBox.Show(Vel.ToString());

            Velocity *= Speed;
        }
    }
}
