﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
#if XNA
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endif

namespace VillainmadEngine
{
    public partial class GameEntity
    {
        private Rectangle ReRect(Rectangle rect)
        {
            rect.X += (int)Math.Round(Position.X);
            rect.Y += (int)Math.Round(Position.Y);
            return rect;
        }

        #region Draw()
        #region Color, Round
        /// <summary>
        /// Draws the entity at its position.
        /// </summary>
        /// <param name="color">Color to tint the entity.</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void Draw(Color color, bool Round)
        {
            Drawing.Draw(Texture, Round ? Position.Round() : Position, null, color, Angle, null, 1F,
                SpriteEffects.None, BlendState);
        }
        #endregion

        #region Texture, Color, Round
        /// <summary>
        /// Draws the specified texture at the entity's position.
        /// </summary>
        /// <param name="texture">Filename of the texture to use.</param>
        /// <param name="color">Color to tint the texture.</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void Draw(string texture, Color color, bool Round)
        {
            Drawing.Draw(texture, Round ? Position.Round() : Position, null, color, Angle, null, 1F,
                SpriteEffects.None, BlendState);
        }
        #endregion

        #region SrcRectangle, Color, Round
        /// <summary>
        /// Draws the entity's texture at the specified rectangle-position.
        /// </summary>
        /// <param name="SourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">Color to tint the texture.</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void Draw(Rectangle? SourceRectangle, Color color, bool Round)
        {
            Drawing.Draw(Texture, Round ? Position.Round() : Position, SourceRectangle, color, 
                Angle, null, 1F, SpriteEffects.None, BlendState);
        }
        #endregion

        #region Texture, SrcRectangle, Color, Round
        /// <summary>
        /// Draws the specified texture at the entity's position.
        /// </summary>
        /// <param name="texture">Filename of the texture to use.</param>
        /// <param name="SourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">Color to tint the texture.</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void Draw(string texture, Rectangle? SourceRectangle, Color color,
            bool Round)
        {
            Drawing.Draw(texture, Round ? Position.Round() : Position, SourceRectangle, 
                color, Angle, null, 1F, SpriteEffects.None, BlendState);
        }
        #endregion

        #region DestRectangle, SourceRectangle, Color
        /// <summary>
        /// Draws the object's texture with the specified destination-rectangle
        /// (offset by the object's position), optional source-rectangle, and 
        /// color.
        /// </summary>
        /// <param name="DestRectangle">Destination-rectangle</param>
        /// <param name="SourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">Color to tint the texture.</param>
        protected void Draw(Rectangle DestRectangle, Rectangle? SourceRectangle,
            Color color)
        {
            Drawing.Draw(Texture, ReRect(DestRectangle), SourceRectangle, color, Rotation,
                null, SpriteEffects.None, BlendState);
        }
        #endregion

        #region Texture, DestRectangle, SourceRectangle, Color
        /// <summary>
        /// Draws the given texture with the specified destination-rectangle
        /// (offset by the object's position), optional source-rectangle, and 
        /// color.
        /// </summary>
        /// <param name="texture">Filename of the texture to use.</param>
        /// <param name="DestRectangle">Destination-rectangle</param>
        /// <param name="SourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">Color to tint the texture.</param>
        protected void Draw(string texture, Rectangle DestRectangle, 
            Rectangle? SourceRectangle, Color color)
        {
            Drawing.Draw(texture, ReRect(DestRectangle), SourceRectangle, color);
        }
        #endregion

        #region Texture DestRect SourceRect, color, rotation, origin, effects, blend
        /// <summary>
        /// Draws the entity with the specified texture, destination rectangle, 
        /// optional source rectangle, color, angle, optional origin, effects, 
        /// and blendstate.
        /// </summary>
        /// <param name="texture">Filename of the texture to use.</param>
        /// <param name="destRectangle">Destination rectangle of the sprite, offset
        /// by the entity's position.</param>
        /// <param name="sourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">The color to tint the entity.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the sprite around its center.</param>
        /// <param name="origin">The sprite's origin/offset from the upper-left
        /// corner. If null, uses the center of the texture/rectangle.</param>
        /// <param name="effects">Effects to apply.</param>
        /// <param name="blend">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        protected void Draw(string texture, Rectangle destRectangle,
            Rectangle? sourceRectangle, Color color, float rotation, Vector2? origin,
            SpriteEffects effects, BlendState blend)
        {
            Drawing.Draw(texture, ReRect(destRectangle), sourceRectangle, color,
                rotation, origin, effects, blend);
        }
        #endregion

        #region Texture SrcRect Color Rot Origin FloatScale Effects Blend Round
        /// <summary>
        /// Draws the entity with the specified texture, optional source rectangle,
        /// color, angle, optional origin, scale, effects, and blendstate.
        /// </summary>
        /// <param name="texture">Filename of the texture to use.</param>
        /// <param name="sourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">The color to tint the sprite.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the sprite around its center.</param>
        /// <param name="origin">The sprite's origin/offset from the upper-left
        /// corner. If null, uses the center of the texture.</param>
        /// <param name="scale">Scale factor; put 1F for full size.</param>
        /// <param name="effects">Effects to apply.</param>
        /// <param name="blend">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void Draw(string texture, Rectangle? sourceRectangle, Color color,
            float rotation, Vector2? origin, float scale, SpriteEffects effects,
            BlendState blend, bool Round)
        {
            Drawing.Draw(texture, Round ? Position.Round() : Position, 
                sourceRectangle, color, rotation, origin, 
                scale, effects, blend);
        }
        #endregion

        #region Texture SrcRect Color Rot Origin VectorScale Effects Blend
        /// <summary>
        /// Draws the entity with the specified texture, optional source rectangle,
        /// color, angle, optional origin, scale, effects, and blendstate.
        /// </summary>
        /// <param name="texture">Filename of the texture to use.</param>
        /// <param name="sourceRectangle">Source rectangle of the sprite on the
        /// texture. If null, uses the entire texture.</param>
        /// <param name="color">The color to tint the sprite.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the sprite around its center.</param>
        /// <param name="origin">The sprite's origin/offset from the upper-left
        /// corner. If null, uses the center of the texture.</param>
        /// <param name="scale">Scale factor; put 1F for full size.</param>
        /// <param name="effects">Effects to apply.</param>
        /// <param name="blend">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void Draw(string texture, Rectangle? sourceRectangle, Color color,
            float rotation, Vector2? origin, Vector2 scale, SpriteEffects effects,
            BlendState blend, bool Round)
        {
            Drawing.Draw(texture, Round ? Position.Round() : Position, 
                sourceRectangle, color, rotation, origin,
                scale, effects, blend);
        }
        #endregion
        #endregion

        #region Animation
        #region Animation, color
        /// <summary>
        /// Draws the specified animation at the entity's position.
        /// </summary>
        /// <param name="animation">The animation to draw.</param>
        /// <param name="color">Color to tint the animation.</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void DrawAnimation(Animation animation, Color color, bool Round)
        {
            DrawAnimation(animation, color, true);
        }
        #endregion

        #region Animation, Color, StartAtCreation
        /// <summary>
        /// Draws the specified animation at the entity's position.
        /// </summary>
        /// <param name="animation">The animation to draw.</param>
        /// <param name="color">Color to tint the animation.</param>
        /// <param name="StartAtCreation">If true, the animation starts from the entity's
        /// creation-time; false to start at the stage's creation.</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void DrawAnimation(Animation animation, Color color, bool StartAtCreation,
            bool Round)
        {
            animation.Draw(StartAtCreation ? Age : DataStealer.ActiveStage.Frame,
                Round ? Position.Round() : Position, 
                color, Rotation, SpriteEffects.None, BlendState);
        }
        #endregion

        #region Animation, Color, Rotation, SpriteEffects, BlendState
        /// <summary>
        /// Draws the given animation with the specified color, rotation, sprite-effects,
        /// and blendstate.
        /// </summary>
        /// <param name="animation">The animation to draw.</param>
        /// <param name="color">Color to tint the animation.</param>
        /// <param name="StartAtCreation">If true, the animation starts from the entity's
        /// creation-time; false to start at the stage's creation.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the frame around its center.</param>
        /// <param name="spriteEffects">Effects to apply.</param>
        /// <param name="blendState">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void DrawAnimation(Animation animation, Color color, bool StartAtCreation, 
            float rotation, SpriteEffects spriteEffects, BlendState blendState, bool Round)
        {
            animation.Draw(StartAtCreation ? Age : DataStealer.ActiveStage.Frame,
                Round ? Position.Round() : Position,
                color,rotation, spriteEffects, blendState);
        }
        #endregion

        #region Animation, Color, Rotation, FloatScale, SpriteEffects, BlendState
        /// <summary>
        /// Draws the given animation with the specified color, rotation, sprite-effects,
        /// and blendstate.
        /// </summary>
        /// <param name="animation">The animation to draw.</param>
        /// <param name="color">Color to tint the animation.</param>
        /// <param name="StartAtCreation">If true, the animation starts from the entity's
        /// creation-time; false to start at the stage's creation.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the frame around its center.</param>
        /// <param name="scale">Scale factor; 1F = full size.</param>
        /// <param name="spriteEffects">Effects to apply.</param>
        /// <param name="blendState">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void DrawAnimation(Animation animation, Color color, bool StartAtCreation,
            float rotation, float scale, SpriteEffects spriteEffects, BlendState blendState,
            bool Round)
        {
            animation.Draw(StartAtCreation ? Age : DataStealer.ActiveStage.Frame,
                Round ? Position.Round() : Position,
                color, rotation, scale, spriteEffects, blendState);
        }
        #endregion

        #region Animation, Color, Rotation, VectScale, SpriteEffects, BlendState
        /// <summary>
        /// Draws the given animation with the specified color, rotation, sprite-effects,
        /// and blendstate.
        /// </summary>
        /// <param name="animation">The animation to draw.</param>
        /// <param name="color">Color to tint the animation.</param>
        /// <param name="StartAtCreation">If true, the animation starts from the entity's
        /// creation-time; false to start at the stage's creation.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the frame around its center.</param>
        /// <param name="scale">Scale factor; Vector.One = full size.</param>
        /// <param name="spriteEffects">Effects to apply.</param>
        /// <param name="blendState">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        /// <param name="Round">If true, the entity's position is rounded to
        /// the nearest whole-number X and Y coordinates.</param>
        protected void DrawAnimation(Animation animation, Color color, bool StartAtCreation,
            float rotation, Vector2 scale, SpriteEffects spriteEffects, BlendState blendState,
            bool Round)
        {
            animation.Draw(StartAtCreation ? Age : DataStealer.ActiveStage.Frame,
                Round ? Position.Round() : Position,
                color, rotation, scale, spriteEffects, blendState);
        }
        #endregion
        #endregion
    }
}
