﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    /// <summary>
    /// A command which can be enqueued within a menu.
    /// </summary>
    public class QueuedCommand
    {
        private Action _command = null;
        private bool _cancelled = false;
        private int _maxAge = -1;

        /// <summary>
        /// True if the command has been cancelled; false otherwise.
        /// </summary>
        public bool Cancelled { get { return _cancelled; } }

        /// <summary>
        /// Invokes the action.
        /// </summary>
        public void Act()
        {
            _command.Invoke();
            this.Cancel();
        }

        /// <summary>
        /// Checks whether this command needs attention when going through the list,
        /// either because it has been cancelled or because it is ready to execute.
        /// </summary>
        /// <param name="Frame">Checks the Frame of the current parent-item.</param>
        /// <returns>True if the command has either been cancelled or is ready to fire;
        /// false otherwis.</returns>
        public bool Attention(int Frame)
        {
            return (Cancelled || TargetFrame <= Frame);
        }

        /// <summary>
        /// The frame at which the command will be fired.
        /// </summary>
        public int TargetFrame
        {
            get { return _maxAge; }
            set { if (!Cancelled) _maxAge = value; }
        }

        /// <summary>
        /// Creates a new command. Don't use this directly, use Menu.Enqueue()
        /// instead!
        /// </summary>
        /// <param name="MaxAge">The frame at which it will be delayed.</param>
        /// <param name="Command">The command to perform, in the form of
        /// delegate(){ &lt;action-list&gt; }</param>
        public QueuedCommand(int MaxAge, Action Command)
        {
            this._command = Command;
            this._maxAge = MaxAge;
        }

        /// <summary>
        /// Cancels the command from being fired.
        /// </summary>
        public void Cancel()
        {
            _cancelled = true;
            _maxAge = -1;
            _command = delegate() { };
        }

        /// <summary>
        /// Returns a string representation of the command.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            if (Cancelled) return "Cancelled command";
            return "MaxAge: " + TargetFrame + " / Command: " + _command;
        }
    }
}
