﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

namespace VillainmadEngine
{
    public static partial class Sound
    {
        private static List<NamedInstance> BGMList = new List<NamedInstance>();

        private static SoundEffectInstance CurBGM
        {
            get
            {
                if (BGMList.Count == 0) return null;
                return BGMList[BGMList.Count - 1].Instance;
            }
        }

        /// <summary>
        /// Plays the currently-playing background music.
        /// </summary>
        public static void BGMPlay()
        {
            if (CurBGM == null) return;
            if (CurBGM.State != SoundState.Playing) CurBGM.Play();
        }

        /// <summary>
        /// Stops the currently-playing background music and removes it from the list.
        /// </summary>
        public static void BGMEnd()
        {
            if (CurBGM == null) return;
            RemoveBGM(BGMList.Count);
        }

        /// <summary>
        /// Pauses the currently-playing background music so that it can be resumed later.
        /// </summary>
        public static void BGMPause()
        {
            if (CurBGM == null) return;
            CurBGM.Pause();
        }

        /// <summary>
        /// Stops the currently-playing background music, without removing it from the list.
        /// </summary>
        public static void BGMStop()
        {
            if (CurBGM == null) return;
            CurBGM.Stop();
        }

        /// <summary>
        /// Adds a new background music track to the list, stopping and removing the 
        /// previous BGM if applicable.
        /// If another background music track is present with the same name, it will be 
        /// removed.
        /// </summary>
        /// <param name="Filename">Filename of the sound in the "sfx/" folder in 
        /// Content. Does not include extension.</param>
        public static void BGMStart(string Filename)
        {
            BGMStart(Filename, BGMStopOption.StopAndRemove, 1.0F, 0F, 0F);
        }

        private static void RemoveBGM(int i)
        {
            BGMList[i].Instance.Stop();
            BGMList[i].Dispose();
            BGMList.RemoveAt(i);
        }

        /// <summary>
        /// Adds a new BGM to the list, with options for what to do to the previous BGM
        /// if applicable.
        /// If another background music track is present with the same name, it will be 
        /// removed.
        /// </summary>
        /// <param name="Filename">Filename of the sound in the "sfx/" folder in 
        /// Content. Does not include extension.</param>
        /// <param name="options">The options to apply to the previous BGM, if
        /// any.</param>
        /// <param name="Volume">Volume, ranging from 0.0F (silence) to 1.0F (full
        /// volume).</param>
        /// <param name="Pitch">Pitch adjustment, ranging from -1.0F (down one ocative)
        /// to 1.0F (up one ocative). 0.0F is normal pitch.</param>
        /// <param name="Pan">Panning, ranging from -1.0F (full left) to 1.0F (full
        /// right). 0.0F is centered.</param>
        public static void BGMStart(string Filename, BGMStopOption options, float Volume,
            float Pitch, float Pan)
        {
            SoundEffectCheck(Filename);
            switch (options)
            {
                case BGMStopOption.Pause:
                    BGMPause();
                    break;
                case BGMStopOption.Stop:
                    BGMStop();
                    break;
                case BGMStopOption.StopAndRemove:
                    BGMEnd();
                    break;
            }

            for (int i = 0; i < BGMList.Count; i++)
            {
                while (i < BGMList.Count &&
                    BGMList[i].Name == Filename)
                    RemoveBGM(i);
            }

            SoundEffectInstance NewSound =
                DataStealer.SoundEffectList[Filename].CreateInstance();

            NewSound.Pan = Pan;
            NewSound.Pitch = Pitch;
            NewSound.Volume = Volume;
            
            NewSound.IsLooped = true;

            BGMList.Add(new NamedInstance(NewSound, Filename));
            BGMPlay();
        }

        /// <summary>
        /// Volume of the current background music, ranging from 0.0F
        /// (silent) to 1.0F (fill volume).
        /// Default is 1.0F.
        /// Will cause errors if no BGM is currently present.
        /// </summary>
        public static float BGMVolume
        {
            get { return CurBGM.Volume; }
            set { CurBGM.Volume = value; }
        }

        /// <summary>
        /// Pitch of the current background music, ranging from -1.0F
        /// (one ocative down) to 1.0F (one ocative up). Default is
        /// 0.0F (normal pitch).
        /// Will cause errors if no BGM is currently present.
        /// </summary>
        public static float BGMPitch
        {
            get { return CurBGM.Pitch; }
            set { CurBGM.Pitch = value; }
        }

        /// <summary>
        /// Pan of the current background music, ranging from -1.0F
        /// (full left) to 1.0F (full right). Default is 0.0F (centered).
        /// Will cause errors if no BGM is currently present.
        /// </summary>
        public static float BGMPan
        {
            get { return CurBGM.Pan; }
            set { CurBGM.Pan = value; }
        }
    }
}
