﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace VillainmadEngine
{
    public partial class Stage
    {
        private void ExtCheck(GameEntity Entity)
        {
            if (Extant(Entity)) throw new ArgumentException("That entity already exists!");
        }

        /// <summary>
        /// Adds the given GameEntity to the list with the specified position and velocity.
        /// </summary>
        /// <param name="Entity">The entity to use, i.e. "new GameEntityType(...)"</param>
        /// <param name="Position">The starting position of the GameEntity.</param>
        /// <param name="Velocity">The starting velocity of the GameEntity.</param>
        /// <returns>The entity which was just added.</returns>
        public GameEntity AddEntity(GameEntity Entity, Vector2 Position,
            Vector2 Velocity)
        {
            ExtCheck(Entity);

            Entity.Position = Position;
            Entity.Velocity = Velocity;
            InstallEntity(Entity);
            return Entity;
        }

        /// <summary>
        /// Adds the given GameEntity to the list with the specified position, velocity,
        /// and angle of rotation.
        /// </summary>
        /// <param name="Entity">The entity to use, i.e. "new GameEntityType(...)"</param>
        /// <param name="Position">The starting position of the GameEntity.</param>
        /// <param name="Velocity">The starting velocity of the GameEntity.</param>
        /// <param name="Angle">The entity's starting angle of rotation, in radians.</param>
        /// <returns>The entity which was just added.</returns>
        public GameEntity AddEntity(GameEntity Entity, Vector2 Position,
            Vector2 Velocity, float Angle)
        {
            ExtCheck(Entity);

            Entity.Position = Position;
            Entity.Velocity = Velocity;
            Entity.Angle = Angle;

            InstallEntity(Entity);
            return Entity;
        }

        /// <summary>
        /// Adds the given GameEntity to the list with the specified position, velocity,
        /// angle of rotation, and rotation speed.
        /// </summary>
        /// <param name="Entity">The entity to use, i.e. "new GameEntityType(...)"</param>
        /// <param name="Position">The starting position of the GameEntity.</param>
        /// <param name="Velocity">The starting velocity of the GameEntity.</param>
        /// <param name="Angle">The entity's starting angle of rotation, in radians.</param>
        /// <param name="Rotation">The rate at which the GameEntity will rotate
        /// every frame, in radians. If negative, it will rotate along with its 
        /// Velocity.</param>
        /// <returns>The entity which was just added.</returns>
        public GameEntity AddEntity(GameEntity Entity, Vector2 Position,
            Vector2 Velocity, float Angle, float Rotation)
        {
            ExtCheck(Entity);

            Entity.Position = Position;
            Entity.Velocity = Velocity;
            Entity.Angle = Angle;
            Entity.Rotation = Rotation;

            InstallEntity(Entity);
            return Entity;
        }


        /// <summary>
        /// Creates a new GameEntity of the given type with the specified 
        /// position and velocity.
        /// NOTE: If Position and Velocity are set in the constructor,
        /// they will be overwritten.
        /// </summary>
        /// <typeparam name="T">Type of the GameEntity. Must have a constructor
        /// that takes no arguments.</typeparam>
        /// <param name="Position">The starting position of the GameEntity.</param>
        /// <param name="Velocity">The starting velocity of the GameEntity.</param>
        /// <returns>The newly-created entity.</returns>
        public GameEntity AddEntity<T>(Vector2 Position,
            Vector2 Velocity)
            where T : GameEntity, new()
        {
            return AddEntity(new T(), Position, Velocity);
        }

        /// <summary>
        /// Creates a new GameEntity of the given type with the specified 
        /// position and velocity.
        /// NOTE: If Position and Velocity are set in the constructor,
        /// they will be overwritten.
        /// </summary>
        /// <typeparam name="T">Type of the GameEntity. Must have a constructor
        /// that takes no arguments.</typeparam>
        /// <param name="Position">The starting position of the GameEntity.</param>
        /// <param name="Velocity">The starting velocity of the GameEntity.</param>
        /// <param name="Angle">The entity's starting angle of rotation, in radians.</param>
        /// <returns>The newly-created entity.</returns>
        public GameEntity AddEntity<T>(Vector2 Position,
            Vector2 Velocity, float Angle)
            where T : GameEntity, new()
        {
            return AddEntity(new T(), Position, Velocity, Angle);
        }

        /// <summary>
        /// Creates a new GameEntity of the given type with the specified 
        /// position and velocity.
        /// NOTE: If Position and Velocity are set in the constructor,
        /// they will be overwritten.
        /// </summary>
        /// <typeparam name="T">Type of the GameEntity. Must have a constructor
        /// that takes no arguments.</typeparam>
        /// <param name="Position">The starting position of the GameEntity.</param>
        /// <param name="Velocity">The starting velocity of the GameEntity.</param>
        /// <param name="Angle">The entity's starting angle of rotation, in radians.</param>
        /// <param name="Rotation">The rate at which the GameEntity will rotate
        /// every frame, in radians. If negative, it will rotate along with its 
        /// Velocity.</param>
        /// <returns>The newly-created entity.</returns>
        public GameEntity AddEntity<T>(Vector2 Position,
            Vector2 Velocity, float Angle, float Rotation)
            where T : GameEntity, new()
        {
            return AddEntity(new T(), Position, Velocity, Angle, Rotation);
        }

        private void InstallEntity(GameEntity Entity)
        {
            EntityArray[Entity.Layer].Add(Entity);
            Entity.OnStart();
        }

        /// <summary>
        /// Returns whether or not the given GameEntity is currently in the entity-list
        /// and has not been deleted.
        /// </summary>
        /// <param name="Entity">The GameEntity to check.</param>
        /// <returns>True if the given GameEntity exists; false otherwise.</returns>
        public bool Extant(GameEntity Entity)
        {
            return EntityArray[Entity.Layer].Contains(Entity) && !Entity.Deleted;
        }

        /// <summary>
        /// Removes an entity from the game-list.
        /// </summary>
        /// <param name="Entity">The GameEntity to remove.</param>
        public void RemoveEntity(GameEntity Entity)
        {
            if (!EntityArray[Entity.Layer].Contains(Entity)) return;

            EntityArray[Entity.Layer].Remove(Entity);
            Entity.Remove();
        }
    }
}
