﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace VillainmadEngine
{
    /// <summary>
    /// Some useful utilities and extension-methods.
    /// </summary>
    public static class Utilities
    {
        /// <summary>
        /// Calculates the angle of the vector, in radians.
        /// </summary>
        /// <param name="val">The vector</param>
        /// <returns>The angle of the vector in radians.</returns>
        public static float Angle(this Vector2 val)
        {
            return (float)Math.Atan2(val.Y, val.X);
        }

        /// <summary>
        /// Returns the center of the texture.
        /// </summary>
        /// <param name="texture">The texture</param>
        /// <returns>A Vector2 with X and Y equal to half of the texture's width 
        /// and height.</returns>
        public static Vector2 Center(this Texture2D texture)
        {
            return new Vector2(texture.Width / 2, texture.Height / 2);
        }

        /// <summary>
        /// Creates a new Vector2 from the specified length and angle.
        /// </summary>
        /// <param name="radians">The angle, in radians.</param>
        /// <param name="length">The length of the intended vector.</param>
        /// <returns>The new Vector2.</returns>
        public static Vector2 FromAngle(float radians, float length)
        {
            float x, y;
            x = (float)(length * Math.Cos(radians));
            y = (float)(length * Math.Sin(radians));
            return new Vector2(x, y);
        }

        /// <summary>
        /// Creates a new normalized Vector2 from the specified angle.
        /// </summary>
        /// <param name="radians">The angle, in radians.</param>
        /// <returns>The new Vector2, 1 unit in length.</returns>
        public static Vector2 FromAngle(float radians)
        {
            return FromAngle(radians, 1F);
        }

        /// <summary>
        /// Creates a new Vector2 from the specified length and angle.
        /// </summary>
        /// <param name="length">The length of the intended vector.</param>
        /// <param name="degrees">The angle, in degrees.</param>
        /// <returns>The new Vector2.</returns>
        public static Vector2 FromAngleDeg(float degrees, float length)
        {
            return FromAngle(MathHelper.ToRadians(degrees), length);
        }

        /// <summary>
        /// Rotates a vector by the specified number of radians,
        /// maintaining the length.
        /// </summary>
        /// <param name="vector">The input vector.</param>
        /// <param name="radians">The angle to add to it in radians.</param>
        /// <returns>The new Vector2.</returns>
        public static Vector2 AddRadians(this Vector2 vector, float radians)
        {
            return FromAngle(vector.Angle() + radians, vector.Length());
        }

        /// <summary>
        /// Rotates a vector by the specified number of degrees,
        /// maintaining the length.
        /// </summary>
        /// <param name="vector">The Vector2 to rotate.</param>
        /// <param name="degrees">The angle to add to it in degrees.</param>
        /// <returns>The new Vector2.</returns>
        public static Vector2 AddDegrees(this Vector2 vector, float degrees)
        {
            return AddRadians(vector, MathHelper.ToRadians(degrees));
        }

        /// <summary>
        /// Rounds a vector's X and Y coordinates to the nearest whole number.
        /// </summary>
        /// <param name="vector">The Vector2 to round.</param>
        /// <returns>A Vector2 with the X and Y coordinates rounded.</returns>
        public static Vector2 Round(this Vector2 vector)
        {
            return new Vector2((float)Math.Round(vector.X),
                (float)Math.Round(vector.Y));
        }
    }
}
